@@ -1,25 +1,27 @@
 # This file is Copyright 2019 Volatility Foundation and licensed under the Volatility Software License 1.0
 # which is available at https://www.volatilityfoundation.org/license/vsl-v1.0
 #
-
+import contextlib
+import functools
+import logging
 from typing import Generator, Iterable, Optional, Set, Tuple
 
-import logging
-
-from volatility3.framework import constants, objects, renderers
-from volatility3.framework import exceptions, interfaces
+from volatility3.framework import constants, exceptions, interfaces, objects
 from volatility3.framework.objects import utility
 from volatility3.framework.renderers import conversion
 from volatility3.framework.symbols import generic
 
 vollog = logging.getLogger(__name__)
 
-class proc(generic.GenericIntelProcess):
 
+class proc(generic.GenericIntelProcess):
     def get_task(self):
         return self.task.dereference().cast("task")
 
-    def add_process_layer(self, config_prefix: str = None, preferred_name: str = None) -> Optional[str]:
+    @functools.lru_cache
+    def add_process_layer(
+        self, config_prefix: Optional[str] = None, preferred_name: Optional[str] = None
+    ) -> Optional[str]:
         """Constructs a new layer based on the process's DTB.
 
         Returns the name of the Layer or None.
@@ -27,42 +29,53 @@ class proc(generic.GenericIntelProcess):
         parent_layer = self._context.layers[self.vol.layer_name]
 
         if not isinstance(parent_layer, interfaces.layers.TranslationLayerInterface):
-            raise TypeError("Parent layer is not a translation layer, unable to construct process layer")
+            raise TypeError(
+                "Parent layer is not a translation layer, unable to construct process layer"
+            )
 
         try:
             dtb = self.get_task().map.pmap.pm_cr3
         except exceptions.InvalidAddressException:
+            # Bail out because we couldn't find the DTB
             return None
 
         if preferred_name is None:
             preferred_name = self.vol.layer_name + f"_Process{self.p_pid}"
 
         # Add the constructed layer and return the name
-        return self._add_process_layer(self._context, dtb, config_prefix, preferred_name)
+        return self._add_process_layer(
+            self._context, dtb, config_prefix, preferred_name
+        )
 
     def get_map_iter(self) -> Iterable[interfaces.objects.ObjectInterface]:
         try:
             task = self.get_task()
-        except exceptions.InvalidAddressException:
-            return
-
-        try:
             current_map = task.map.hdr.links.next
         except exceptions.InvalidAddressException:
-            return
+            return None
 
         seen: Set[int] = set()
 
         for i in range(task.map.hdr.nentries):
-            if (not current_map or
-                current_map.vol.offset in seen or
-                not self._context.layers[task.vol.native_layer_name].is_valid(current_map.dereference().vol.offset, current_map.dereference().vol.size)):
-
-                vollog.log(constants.LOGLEVEL_VVV, "Breaking process maps iteration due to invalid state.")
+            if (
+                not current_map
+                or current_map.vol.offset in seen
+                or not self._context.layers[task.vol.native_layer_name].is_valid(
+                    current_map.dereference().vol.offset,
+                    current_map.dereference().vol.size,
+                )
+            ):
+                vollog.log(
+                    constants.LOGLEVEL_VVV,
+                    "Breaking process maps iteration due to invalid state.",
+                )
                 break
 
             # ZP_POISON value used to catch programming errors
-            if current_map.links.start == 0xdeadbeefdeadbeef or current_map.links.end == 0xdeadbeefdeadbeef:
+            if (
+                current_map.links.start == 0xDEADBEEFDEADBEEF
+                or current_map.links.end == 0xDEADBEEFDEADBEEF
+            ):
                 break
 
             yield current_map
@@ -75,11 +88,12 @@ class proc(generic.GenericIntelProcess):
     # the fix for linux was to call int() so that we were not returning vol objects.
     # I call int() on these and the code works nearly 1-1 with the linux one so I am very confused
     ######
-    def get_process_memory_sections(self,
-                                    context: interfaces.context.ContextInterface,
-                                    config_prefix: str,
-                                    rw_no_file: bool = False) -> \
-            Generator[Tuple[int, int], None, None]:
+    def get_process_memory_sections(
+        self,
+        context: interfaces.context.ContextInterface,
+        config_prefix: str,
+        rw_no_file: bool = False,
+    ) -> Generator[Tuple[int, int], None, None]:
         """Returns a list of sections based on the memory manager's view of
         this task's virtual memory."""
         for vma in self.get_map_iter():
@@ -87,7 +101,10 @@ class proc(generic.GenericIntelProcess):
             end = int(vma.links.end)
 
             if rw_no_file:
-                if vma.get_perms() != "rw" or vma.get_path(context, config_prefix) != "":
+                if (
+                    vma.get_perms() != "rw"
+                    or vma.get_path(context, config_prefix) != ""
+                ):
                     if vma.get_special_path() != "[heap]":
                         continue
 
@@ -95,17 +112,14 @@ class proc(generic.GenericIntelProcess):
 
 
 class fileglob(objects.StructType):
-
     def get_fg_type(self):
         ret = None
 
         if self.has_member("fg_type"):
             ret = self.fg_type
         elif self.fg_ops != 0:
-            try:
+            with contextlib.suppress(exceptions.InvalidAddressException):
                 ret = self.fg_ops.fo_type
-            except exceptions.InvalidAddressException:
-                pass
 
         if ret:
             ret = str(ret.description).replace("DTYPE_", "")
@@ -114,7 +128,6 @@ class fileglob(objects.StructType):
 
 
 class vm_map_object(objects.StructType):
-
     def get_map_object(self):
         if self.has_member("vm_object"):
             return self.vm_object
@@ -125,31 +138,38 @@ class vm_map_object(objects.StructType):
 
 
 class vnode(objects.StructType):
-
     def _do_calc_path(self, ret, vnodeobj, vname):
         if vnodeobj is None:
-            return
+            return None
 
         if vname:
             try:
                 ret.append(utility.pointer_to_string(vname, 255))
             except exceptions.InvalidAddressException:
-                return
+                return None
 
         if int(vnodeobj.v_flag) & 0x000001 != 0 and int(vnodeobj.v_mount) != 0:
             if int(vnodeobj.v_mount.mnt_vnodecovered) != 0:
-                self._do_calc_path(ret, vnodeobj.v_mount.mnt_vnodecovered, vnodeobj.v_mount.mnt_vnodecovered.v_name)
+                self._do_calc_path(
+                    ret,
+                    vnodeobj.v_mount.mnt_vnodecovered,
+                    vnodeobj.v_mount.mnt_vnodecovered.v_name,
+                )
         else:
             try:
                 parent = vnodeobj.v_parent
                 parent_name = parent.v_name
             except exceptions.InvalidAddressException:
-                return
+                return None
 
             self._do_calc_path(ret, parent, parent_name)
 
     def full_path(self):
-        if self.v_flag & 0x000001 != 0 and self.v_mount != 0 and self.v_mount.mnt_flag & 0x00004000 != 0:
+        if (
+            self.v_flag & 0x000001 != 0
+            and self.v_mount != 0
+            and self.v_mount.mnt_flag & 0x00004000 != 0
+        ):
             ret = b"/"
         else:
             elements = []
@@ -169,7 +189,6 @@ class vnode(objects.StructType):
 
 
 class vm_map_entry(objects.StructType):
-
     def is_suspicious(self, context, config_prefix):
         """Flags memory regions that are mapped rwx or that map an executable
         not back from a file on disk."""
@@ -189,7 +208,7 @@ class vm_map_entry(objects.StructType):
         permask = "rwx"
         perms = ""
 
-        for (ctr, i) in enumerate([1, 3, 5]):
+        for ctr, i in enumerate([1, 3, 5]):
             if (self.protection & i) == i:
                 perms = perms + permask[ctr]
             else:
@@ -201,7 +220,7 @@ class vm_map_entry(objects.StructType):
         if self.has_member("alias"):
             ret = int(self.alias)
         else:
-            ret = int(self.vme_offset) & 0xfff
+            ret = int(self.vme_offset) & 0xFFF
 
         return ret
 
@@ -220,7 +239,7 @@ class vm_map_entry(objects.StructType):
     def get_path(self, context, config_prefix):
         node = self.get_vnode(context, config_prefix)
 
-        if type(node) == str and node == "sub_map":
+        if type(node) is str and node == "sub_map":
             ret = node
         elif node:
             path = []
@@ -302,9 +321,11 @@ class vm_map_entry(objects.StructType):
                 break
 
         if found:
-            vpager = context.object(config_prefix + constants.BANG + "vnode_pager",
-                                    layer_name = vnode_object.vol.native_layer_name,
-                                    offset = vnode_object.pager)
+            vpager = context.object(
+                config_prefix + constants.BANG + "vnode_pager",
+                layer_name=vnode_object.vol.native_layer_name,
+                offset=vnode_object.pager,
+            )
             ret = vpager.vnode_handle
         else:
             ret = None
@@ -313,7 +334,6 @@ class vm_map_entry(objects.StructType):
 
 
 class socket(objects.StructType):
-
     def get_inpcb(self):
         try:
             ret = self.so_pcb.dereference().cast("inpcb")
@@ -371,10 +391,20 @@ class socket(objects.StructType):
 
 
 class inpcb(objects.StructType):
-
     def get_tcp_state(self):
-        tcp_states = ("CLOSED", "LISTEN", "SYN_SENT", "SYN_RECV", "ESTABLISHED", "CLOSE_WAIT", "FIN_WAIT1", "CLOSING",
-                      "LAST_ACK", "FIN_WAIT2", "TIME_WAIT")
+        tcp_states = (
+            "CLOSED",
+            "LISTEN",
+            "SYN_SENT",
+            "SYN_RECV",
+            "ESTABLISHED",
+            "CLOSE_WAIT",
+            "FIN_WAIT1",
+            "CLOSING",
+            "LAST_ACK",
+            "FIN_WAIT2",
+            "TIME_WAIT",
+        )
 
         try:
             tcpcb = self.inp_ppcb.dereference().cast("tcpcb")
@@ -408,14 +438,18 @@ class inpcb(objects.StructType):
 
     def get_ipv6_info(self):
         try:
-            lip = self.inp_dependladdr.inp6_local.member(attr = '__u6_addr').member(attr = '__u6_addr32')
+            lip = self.inp_dependladdr.inp6_local.member(attr="__u6_addr").member(
+                attr="__u6_addr32"
+            )
         except exceptions.InvalidAddressException:
             return None
 
         lport = self.inp_lport
 
         try:
-            rip = self.inp_dependfaddr.inp6_foreign.member(attr = '__u6_addr').member(attr = '__u6_addr32')
+            rip = self.inp_dependfaddr.inp6_foreign.member(attr="__u6_addr").member(
+                attr="__u6_addr32"
+            )
         except exceptions.InvalidAddressException:
             return None
 
@@ -425,12 +459,13 @@ class inpcb(objects.StructType):
 
 
 class queue_entry(objects.StructType):
-
-    def walk_list(self,
-                  list_head: interfaces.objects.ObjectInterface,
-                  member_name: str,
-                  type_name: str,
-                  max_size: int = 4096) -> Iterable[interfaces.objects.ObjectInterface]:
+    def walk_list(
+        self,
+        list_head: interfaces.objects.ObjectInterface,
+        member_name: str,
+        type_name: str,
+        max_size: int = 4096,
+    ) -> Iterable[interfaces.objects.ObjectInterface]:
         """
         Walks a queue in a smear-aware and smear-resistant manner
 
@@ -455,30 +490,32 @@ class queue_entry(objects.StructType):
 
         seen = set()
 
-        for attr in ['next', 'prev']:
-            try:
-                n = getattr(self, attr).dereference().cast(type_name)
-
-                while n is not None and n.vol.offset != list_head:
-                    if n.vol.offset in seen:
+        for attr in ["next", "prev"]:
+            with contextlib.suppress(exceptions.InvalidAddressException):
+                queue_element = getattr(self, attr).dereference().cast(type_name)
+                while (
+                    queue_element is not None
+                    and queue_element.vol.offset != list_head.vol.offset
+                ):
+                    if queue_element.vol.offset in seen:
                         break
 
-                    yield n
+                    yield queue_element
 
-                    seen.add(n.vol.offset)
+                    seen.add(queue_element.vol.offset)
 
                     yielded = yielded + 1
                     if yielded == max_size:
-                        return
+                        return None
 
-                    n = getattr(n.member(attr = member_name), attr).dereference().cast(type_name)
-
-            except exceptions.InvalidAddressException:
-                pass
+                    queue_element = (
+                        getattr(queue_element.member(attr=member_name), attr)
+                        .dereference()
+                        .cast(type_name)
+                    )
 
 
 class ifnet(objects.StructType):
-
     def sockaddr_dl(self):
         if self.has_member("if_lladdr"):
             try:
@@ -487,7 +524,9 @@ class ifnet(objects.StructType):
                 val = None
         else:
             try:
-                val = self.if_addrhead.tqh_first.ifa_addr.dereference().cast("sockaddr_dl")
+                val = self.if_addrhead.tqh_first.ifa_addr.dereference().cast(
+                    "sockaddr_dl"
+                )
             except exceptions.InvalidAddressException:
                 val = None
 
@@ -496,7 +535,6 @@ class ifnet(objects.StructType):
 
 # this is used for MAC addresses
 class sockaddr_dl(objects.StructType):
-
     def __str__(self):
         ret = ""
 
@@ -520,7 +558,6 @@ class sockaddr_dl(objects.StructType):
 
 
 class sockaddr(objects.StructType):
-
     def get_address(self):
         ip = ""
 
@@ -531,7 +568,9 @@ class sockaddr(objects.StructType):
 
         elif family == 30:  # AF_INET6
             addr_in6 = self.cast("sockaddr_in6")
-            ip = conversion.convert_ipv6(addr_in6.sin6_addr.member(attr = "__u6_addr").member(attr = "__u6_addr32"))
+            ip = conversion.convert_ipv6(
+                addr_in6.sin6_addr.member(attr="__u6_addr").member(attr="__u6_addr32")
+            )
 
         elif family == 18:  # AF_LINK
             addr_dl = self.cast("sockaddr_dl")
@@ -541,7 +580,6 @@ class sockaddr(objects.StructType):
 
 
 class sysctl_oid(objects.StructType):
-
     def get_perms(self) -> str:
         """
         Returns the actions allowed on the node
@@ -559,7 +597,7 @@ class sysctl_oid(objects.StructType):
         checks = [0x80000000, 0x40000000, 0x00800000]
         perms = ["R", "W", "L"]
 
-        for (i, c) in enumerate(checks):
+        for i, c in enumerate(checks):
             if c & self.oid_kind:
                 ret = ret + perms[i]
             else:
@@ -584,9 +622,15 @@ class sysctl_oid(objects.StructType):
 
         Based on sysctl_sysctl_debug_dump_node
         """
-        types = {1: 'CTLTYPE_NODE', 2: 'CTLTYPE_INT', 3: 'CTLTYPE_STRING', 4: 'CTLTYPE_QUAD', 5: 'CTLTYPE_OPAQUE'}
+        types = {
+            1: "CTLTYPE_NODE",
+            2: "CTLTYPE_INT",
+            3: "CTLTYPE_STRING",
+            4: "CTLTYPE_QUAD",
+            5: "CTLTYPE_OPAQUE",
+        }
 
-        ctltype = self.oid_kind & 0xf
+        ctltype = self.oid_kind & 0xF
 
         if 0 < ctltype < 6:
             ret = types[ctltype]
@@ -597,7 +641,6 @@ class sysctl_oid(objects.StructType):
 
 
 class kauth_scope(objects.StructType):
-
     def get_listeners(self):
         for listener in self.ks_listeners:
             if listener != 0 and listener.kll_callback != 0:
