@@ -4,8 +4,7 @@
 
 import datetime
 import logging
-from typing import Optional, Tuple, Union
-
+from typing import Optional, Tuple, Union, List, Dict
 from volatility3.framework import constants, interfaces
 
 vollog = logging.getLogger(__name__)
@@ -19,9 +18,16 @@ class ProducerMetadata(interfaces.symbols.MetadataInterface):
         return self._json_data.get("name", None)
 
     @property
-    def version(self) -> Optional[Tuple[int]]:
+    def version_string(self) -> str:
+        """Returns the ISF file producer's version as a string.
+        If no version is present, an empty string is returned.
+        """
+        return self._json_data.get("version", "")
+
+    @property
+    def version(self) -> Optional[Tuple[int, ...]]:
         """Returns the version of the ISF file producer"""
-        version = self._json_data.get("version", None)
+        version = self.version_string
         if not version:
             return None
         if all(x in "0123456789." for x in version):
@@ -79,5 +85,21 @@ class WindowsMetadata(interfaces.symbols.MetadataInterface):
         return self._json_data.get("pdb", {}).get("age", None)
 
 
-class LinuxMetadata(interfaces.symbols.MetadataInterface):
+class PosixMetadata(interfaces.symbols.MetadataInterface):
+    """Base class to handle metadata of Posix-based ISF sources"""
+
+    def get_types_sources(self) -> List[Optional[Dict]]:
+        """Returns the types sources metadata"""
+        return self._json_data.get("types", [])
+
+    def get_symbols_sources(self) -> List[Optional[Dict]]:
+        """Returns the symbols sources metadata"""
+        return self._json_data.get("symbols", [])
+
+
+class LinuxMetadata(PosixMetadata):
     """Class to handle the metadata from a Linux symbol table."""
+
+
+class MacMetadata(PosixMetadata):
+    """Class to handle the metadata from a Mac symbol table."""
