@@ -16,7 +16,6 @@ from volatility3 import symbols
 from volatility3.framework import constants, contexts, exceptions, interfaces
 from volatility3.framework.automagic import symbol_cache
 from volatility3.framework.configuration import requirements
-from volatility3.framework.configuration.requirements import SymbolTableRequirement
 from volatility3.framework.symbols import intermed
 from volatility3.framework.symbols.windows import pdbconv
 
@@ -36,7 +35,7 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         layer_name: str,
         offset: int,
         symbol_table_class: str = "volatility3.framework.symbols.intermed.IntermediateSymbolTable",
-        config_path: str = None,
+        config_path: Optional[str] = None,
         progress_callback: constants.ProgressCallback = None,
     ) -> Optional[str]:
         """Produces the name of a symbol table loaded from the offset for an MZ header
@@ -94,7 +93,7 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         if not requirements.VersionRequirement.matches_required(
             (1, 0, 0), symbol_cache.SqliteCache.version
         ):
-            vollog.debug(f"Required version of SQLiteCache not found")
+            vollog.debug("Required version of SQLiteCache not found")
             return None
 
         identifiers_path = os.path.join(
@@ -140,7 +139,7 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         requirement_name = interfaces.configuration.path_head(config_path)
 
         # Construct the appropriate symbol table
-        requirement = SymbolTableRequirement(
+        requirement = requirements.SymbolTableRequirement(
             name=requirement_name, description="PDBUtility generated symbol table"
         )
         requirement.construct(context, parent_config_path)
@@ -291,9 +290,7 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
                 break
             except PermissionError:
                 vollog.warning(
-                    "Cannot write necessary symbol file, please check permissions on {}".format(
-                        potential_output_filename
-                    )
+                    f"Cannot write necessary symbol file, please check permissions on {potential_output_filename}"
                 )
                 continue
             finally:
@@ -390,8 +387,8 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         config_path: str,
         layer_name: str,
         pdb_name: str,
-        module_offset: int = None,
-        module_size: int = None,
+        module_offset: Optional[int] = None,
+        module_size: Optional[int] = None,
     ) -> str:
         """Creates symbol table for a module in the specified layer_name.
 
@@ -411,6 +408,12 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         _, symbol_table_name = cls._modtable_from_pdb(
             context, config_path, layer_name, pdb_name, module_offset, module_size
         )
+
+        if symbol_table_name is None:
+            raise exceptions.SymbolSpaceError(
+                f"Symbol table could not be reconstructed for module {pdb_name}"
+            )
+
         return symbol_table_name
 
     @classmethod
@@ -420,8 +423,8 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         config_path: str,
         layer_name: str,
         pdb_name: str,
-        module_offset: int = None,
-        module_size: int = None,
+        module_offset: Optional[int] = None,
+        module_size: Optional[int] = None,
         create_module: bool = False,
     ) -> Tuple[Optional[str], Optional[str]]:
         if module_offset is None:
@@ -441,7 +444,7 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         )
 
         if not guids:
-            raise exceptions.VolatilityException(
+            raise exceptions.SymbolSpaceError(
                 f"Did not find GUID of {pdb_name} in module @ 0x{module_offset:x}!"
             )
 
@@ -480,8 +483,8 @@ class PDBUtility(interfaces.configuration.VersionableInterface):
         config_path: str,
         layer_name: str,
         pdb_name: str,
-        module_offset: int = None,
-        module_size: int = None,
+        module_offset: Optional[int] = None,
+        module_size: Optional[int] = None,
     ) -> str:
         """Creates a module in the specified layer_name based on a pdb name.
 
